<?php

//
// class.cropcanvas.php
// version 1.1.0, 29th July, 2003
//
// Description
//
// This is a class allows you to crop an image in a variety of ways.
// You can crop in an absolute or relative way (to a certain size or
// by a certain size), both as a pixel number or a percentage.  You
// can also save or display the cropped image.  The cropping can be
// done in 9 different positions: top left, top, top right, left,
// centre, right, bottom left, bottom, or bottom right.  Or you can
// crop automatically based on a threshold limit.  The original
// image can be loaded from the file system or from a string (for
// example, data returned from a database.)
//
// Author
//
// Andrew Collington, 2003
// php@amnuts.com, http://php.amnuts.com/
//
// Feedback
//
// http://php.amnuts.com/forums/index.php
// There is message board at the above URL where you can post
// feedback, bug reports, feature requests, etc., for this and
// any of the other scripts located at php.amnuts.com.
//
// Example of use
//
//    require 'class.cropcanvas.php';
//    $cc = new canvasCrop();
//
//    $cc->loadImage('original1.png');
//    $cc->cropBySize(100, 100, ccBOTTOMRIGHT);
//    $cc->saveImage('final1.png');
//
//    $cc->flushImages();
//
//    $cc->loadImage('original2.png');
//    $cc->cropByPercent(15, 50, ccCENTER);
//    $cc->saveImage('final2.jpg', 90);
//
//    $cc->flushImages();
//
//    $cc->loadImage('original3.png');
//    $cc->cropToDimensions(67, 37, 420, 255);
//    $cc->showImage('png');
//

define("ccTOPLEFT",     0);
define("ccTOP",         1);
define("ccTOPRIGHT",    2);
define("ccLEFT",        3);
define("ccCENTRE",      4);
define("ccCENTER",      4);
define("ccRIGHT",       5);
define("ccBOTTOMLEFT",  6);
define("ccBOTTOM",      7);
define("ccBOTTOMRIGHT", 8);

class canvasCrop
{
    var $_imgOrig;
    var $_imgFinal;
    var $_showDebug;
    
    function canvasCrop($debug = FALSE)
    {
        $this->_showDebug = ($debug ? TRUE : FALSE);
    }

    //
    // User called functions
    //

    function loadImage($filename)
    {
        if (!@file_exists($filename))
        {
            $this->_debug('loadImage', "The supplied file name
'$filename' does not point to a readable file.");
            return FALSE;
        }

        $ext  = strtolower($this->_getExtension($filename));
        $func = "imagecreatefrom$ext";

        if (!@function_exists($func))
        {
            $this->_debug('loadImage', "That file cannot be loaded with
the function '$func'.");
            return FALSE;
        }

        $this->_imgOrig = @$func($filename);

        if ($this->_imgOrig == NULL)
        {
            $this->_debug('loadImage', 'The image could not be loaded.');
            return FALSE;
        }
    }

    function loadImageFromString($string)
    {
        $this->_imgOrig = @ImageCreateFromString($string);
        if (!$this->_imgOrig)
        {
            $this->_debug('loadImageFromString', 'The image could not be
loaded.');
            return FALSE;
        }
    }

    function saveImage($filename, $quality = '100')
    {
        if ($this->_imgFinal == NULL)
        {
            $this->_debug('saveImage', 'There is no processed image to
save.');
            return FALSE;
        }

        $ext = strtolower($this->_getExtension($filename));
        $func = "image$ext";

        if (!@function_exists($func))
        {
            $this->_debug('saveImage', "That file cannot be saved with
the function '$func'.");
            return FALSE;
        }

        $saved = FALSE;
        if ($ext == 'png') $saved = $func($this->_imgFinal, $filename);
        if ($ext == 'jpeg') $saved = $func($this->_imgFinal, $filename,
$quality);
        if ($saved == FALSE)
        {
            $this->_debug('saveImage', "Could not save the output file
'$filename' as a $ext.");
            return FALSE;
        }
        else
        {
            return TRUE;
        }
    }

    function showImage($type = 'png', $quality = '100')
    {
        if ($this->_imgFinal == NULL)
        {
            $this->_debug('showImage', 'There is no processed image to
show.');
            return FALSE;
        }
        if ($type == 'png')
        {
            echo @ImagePNG($this->_imgFinal);
            return TRUE;
        }
        else if ($type == 'jpg')
        {
            echo @ImageJPEG($this->_imgFinal, '', $quality);
            return TRUE;
        }
        else
        {
            $this->_debug('showImage', "Could not show the output file
as a $type.");
            return FALSE;
        }
    }

    function cropBySize($x, $y, $position = ccCENTRE)
    {
        $nx = @ImageSX($this->_imgOrig) - $x;
        $ny = @ImageSY($this->_imgOrig) - $y;
        return ($this->_cropSize(-1, -1, $nx, $ny, $position,
'cropBySize'));
    }

    function cropToSize($x, $y, $position = ccCENTRE)
    {
        return ($this->_cropSize(-1, -1, $x, $y, $position, 'cropToSize'));
    }

    function cropToDimensions($sx, $sy, $ex, $ey)
    {
        $nx = abs($ex - $sx);
        $ny = abs($ey - $sy);
        return ($this->_cropSize($sx, $sy, $nx, $ny, $position,
'cropToDimensions'));
    }

    function cropByPercent($percentx, $percenty, $position = ccCENTRE)
    {
        $nx = @ImageSX($this->_imgOrig) - (($percentx / 100) *
@ImageSX($this->_imgOrig));
        $ny = @ImageSY($this->_imgOrig) - (($percenty / 100) *
@ImageSY($this->_imgOrig));
        return ($this->_cropSize(-1, -1, $nx, $ny, $position,
'cropByPercent'));
    }

    function cropToPercent($percentx, $percenty, $position = ccCENTRE)
    {
        $nx = ($percentx / 100) * @ImageSX($this->_imgOrig);
        $ny = ($percenty / 100) * @ImageSY($this->_imgOrig);
        return ($this->_cropSize(-1, -1, $nx, $ny, $position,
'cropByPercent'));
    }

    function cropByAuto($threshold = 254)
    {
        if ($threshold < 0) $threshold = 0;
        if ($threshold > 255) $threshold = 255;

        $sizex = @ImageSX($this->_imgOrig);
        $sizey = @ImageSY($this->_imgOrig);

        $sx = $sy = $ex = $ey = -1;
        for ($y = 0; $y < $sizey; $y++)
        {
            for ($x = 0; $x < $sizex; $x++)
            {
                if ($threshold >=
$this->_getThresholdValue($this->_imgOrig, $x, $y))
                {
                    if ($sy == -1) $sy = $y;
                    else $ey = $y;

                    if ($sx == -1) $sx = $x;
                    else
                    {
                        if ($x < $sx) $sx = $x;
                        else if ($x > $ex) $ex = $x;
                    }
                }
            }
        }
        $nx = abs($ex - $sx);
        $ny = abs($ey - $sy);
        return ($this->_cropSize($sx, $sy, $nx, $ny, ccTOPLEFT,
'cropByAuto'));
    }

    function flushImages()
    {
        @ImageDestroy($this->_imgOrig);
        @ImageDestroy($this->_imgFinal);
        $this->_imgOrig = $this->_imgFinal = NULL;
    }

    //
    // Internal functions
    //

    function _cropSize($ox, $oy, $nx, $ny, $position, $function)
    {
        if ($this->_imgOrig == NULL)
        {
            $this->_debug($function, 'The original image has not been
loaded.');
            return FALSE;
        }
        if (($nx <= 0) || ($ny <= 0))
        {
            $this->_debug($function, 'The image could not be cropped
because the size given is not valid.');
            return FALSE;
        }
        if (($nx >= @ImageSX($this->_imgOrig)) || ($ny >=
@ImageSY($this->_imgOrig)))
        {
            $this->_debug($function, 'The image could not be cropped
because the size given is larger than the original image.');
            return FALSE;
        }
        $this->_imgFinal = @ImageCreateTrueColor($nx, $ny);
        if ($ox == -1 || $oy == -1)
        {
            list($ox, $oy) = $this->_getCopyPosition($nx, $ny, $position);
        }
        @ImageCopyResampled($this->_imgFinal, $this->_imgOrig, 0, 0,
$ox, $oy, $nx, $ny, $nx, $ny);
    }

    function _getCopyPosition($nx, $ny, $position)
    {
        $ox = @ImageSX($this->_imgOrig);
        $oy = @ImageSY($this->_imgOrig);

        switch($position)
        {
            case ccTOPLEFT:
                return array(0, 0);
            case ccTOP:
                return array(ceil(($ox - $nx) / 2), 0);
            case ccTOPRIGHT:
                return array(($ox - $nx), 0);
            case ccLEFT:
                return array(0, ceil(($oy - $ny) / 2));
            case ccCENTRE:
                return array(ceil(($ox - $nx) / 2), ceil(($oy - $ny) / 2));
            case ccRIGHT:
                return array(($ox - $nx), ceil(($oy - $ny) / 2));
            case ccBOTTOMLEFT:
                return array(0, ($oy - $ny));
            case ccBOTTOM:
                return array(ceil(($ox - $nx) / 2), ($oy - $ny));
            case ccBOTTOMRIGHT:
                return array(($ox - $nx), ($oy - $ny));
        }
    }

    function _getThresholdValue($im, $x, $y)
    {
        $rgb = ImageColorAt($im, $x, $y);
        $r = ($rgb >> 16) & 0xFF;
        $g = ($rgb >> 8) & 0xFF;
        $b = $rgb & 0xFF;
        $intensity = ($r + $g + $b) / 3;
        return $intensity;
    }

    function _getExtension($filename)
    {
        $ext  = @strtolower(@substr($filename, (@strrpos($filename, ".")
? @strrpos($filename, ".") + 1 : @strlen($filename)), @strlen($filename)));
        if ($ext == 'jpg') $ext = 'jpeg';
        return $ext;
    }

    function _debug($function, $string)
    {
        if ($this->_showDebug)
        {
            echo "<p><strong style=\"color:#FF0000\">Error in function
$function:</strong> $string</p>\n";
        }
    }

}

?>